@@ -2,7 +2,7 @@
     ooze -- ettercap plugin -- ping a host
 
     Copyright (C) 2001  ALoR <alor@users.sourceforge.net>, NaGA <crwm@freemail.it>
-
+    
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
     the Free Software Foundation; either version 2 of the License, or
@@ -16,30 +16,62 @@
     You should have received a copy of the GNU General Public License
     along with this program; if not, write to the Free Software
     Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+
+    $Id: ooze.c,v 1.7 2002/10/01 21:40:53 alor Exp $
 */
+#ifdef CYGWIN
+   #include <windows.h>
+   #include <winsock2.h> 
+#else
+   #include <sys/time.h>
+   #include <unistd.h>
+#endif
 
 
-#include <unistd.h>
-#include <fcntl.h>
 #include <stdlib.h>
 #include <time.h>
-#include <sys/time.h>
 
 #include "../../src/include/ec_main.h"
+#include "../../src/include/ec_version.h"
 #include "../../src/include/ec_plugins.h"
 #include "../../src/include/ec_inet_structures.h"
 #include "../../src/include/ec_inet.h"
 #include "../../src/include/ec_inet_forge.h"
 
-char Ettercap_Version[] = VERSION;                       // required -- for compatibility
+// protos...
 
-char PlugIn_Info[] = "Ping a host.";
+int Plugin_Init(void *);
+int Plugin_Fini(void *);
+int ooze_function(void *dummy);
 
-short PlugIn_Version = 14;
+// plugin operation
 
-void PlugIn_Start(char**argv)
+struct plugin_ops ooze_ops = {
+   ettercap_version: VERSION,
+   plug_info:        "Ping a host",
+   plug_version:     14,
+   plug_type:        PT_EXT,
+   hook_point:       HOOK_NONE,
+   hook_function:    &ooze_function,
+};
+
+//==================================
+
+int Plugin_Init(void *params)
 {
-   char *buf;
+   return Plugin_Register(params, &ooze_ops);
+}
+
+int Plugin_Fini(void *params)
+{
+   return 0;
+}
+
+// =================================
+
+int ooze_function(void *dummy)
+{
+   struct recv_packet recvpck;
    int sock, MTU, i, reply=0;
    char MyMAC[6], DestMAC[6];
    char MyIP[16];
@@ -52,13 +84,13 @@ void PlugIn_Start(char**argv)
    if (!strcmp(Host_Dest.ip, ""))
    {
       Plugin_Output("\nNo destination host selected !!\n");
-      return;
+      return 0;
    }
 
    if (!strcmp(Host_Dest.ip, Inet_MyIPAddress()))
    {
       Plugin_Output("\nThis plugin doesn't work on myself !!\n");
-      return;
+      return 0;
    }
 
    Plugin_Output("\nHow many ping ? ");
@@ -70,18 +102,19 @@ void PlugIn_Start(char**argv)
    Inet_GetIfaceInfo(Options.netiface, &MTU, NULL, NULL, NULL);
 
    Inet_GetMACfromString(Inet_MyMACAddress(), MyMAC);
-   //Inet_GetMACfromString(Host_Dest.mac, DestMAC);
+
    memcpy (DestMAC, Inet_MacFromIP(inet_addr(Host_Dest.ip)), 6);
 
    strncpy(MyIP,Inet_MyIPAddress(),16);
-
-   buf = Inet_Forge_packet( ETH_HEADER + IP_HEADER + ICMP_HEADER );
-
+   
+   recvpck.buf = Inet_Forge_packet( MTU + ALIGN_ETH_TO_WORD);
+   recvpck.aligned = recvpck.buf + ALIGN_ETH_TO_WORD;
+      
    for (i=0; i<num; i++)
    {
-         Inet_Forge_ethernet( buf, MyMAC, DestMAC, ETH_P_IP );
+         Inet_Forge_ethernet( recvpck.aligned, MyMAC, DestMAC, ETH_P_IP );
 
-         Inet_Forge_ip( buf + ETH_HEADER,
+         Inet_Forge_ip( recvpck.aligned + ETH_HEADER,
                         inet_addr(MyIP),
                         inet_addr(Host_Dest.ip),
                         sizeof(ICMP_header),
@@ -89,22 +122,18 @@ void PlugIn_Start(char**argv)
                         0,
                         IPPROTO_ICMP );
 
-         Inet_Forge_icmp( buf + ETH_HEADER + IP_HEADER, ICMP_ECHO, 0, NULL, 0 );
+         Inet_Forge_icmp( recvpck.aligned + ETH_HEADER + IP_HEADER, ICMP_ECHO, 0, NULL, 0 );
 
 
 
-         Inet_SendRawPacket(sock, buf, ETH_HEADER + IP_HEADER + ICMP_HEADER);
+         Inet_SendRawPacket(sock, recvpck.aligned, ETH_HEADER + IP_HEADER + ICMP_HEADER);
          usleep(1000);
    }
 
 
-   Inet_Forge_packet_destroy( buf );
-
    Plugin_Output("\n%d ICMP echo requests sent to %s. waiting for replies...\n\n", num, Host_Dest.ip);
 
-   buf = Inet_Forge_packet( MTU );
-
-   fcntl(sock, F_SETFL, O_NONBLOCK);
+   Inet_SetNonBlock(sock);
 
    TIME_START;
 
@@ -112,14 +141,14 @@ void PlugIn_Start(char**argv)
    {
       short pkttype, len;
 
-      len = Inet_GetRawPacket(sock, buf, MTU, &pkttype);
+      len = Inet_GetRawPacket(sock, recvpck.aligned, MTU, &pkttype);
 
       TIME_FINISH;
 
       if (len > 0 && pkttype == PACKET_HOST)
       {
 
-         HEther = (ETH_header *) buf;
+         HEther = (ETH_header *) recvpck.aligned;
          if ( ntohs(HEther->type) == ETH_P_IP )
          {
 
@@ -137,7 +166,11 @@ void PlugIn_Start(char**argv)
 
    if (!reply) Plugin_Output("\nNo replies within 3 seconds !! (host could be down)\n");
 
-	close(sock);
+   Inet_CloseRawSock(sock);
+
+   Inet_Forge_packet_destroy(recvpck.buf);
+
+   return 0;
 }
 
 /* EOF */
