@@ -16,23 +16,11 @@
     You should have received a copy of the GNU General Public License
     along with this program; if not, write to the Free Software
     Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+
+    $Id: ec_buffer.c,v 1.8 2002/05/13 20:02:21 alor Exp $
 */
 
-#include <stdio.h>
-#include <stdlib.h>
-#include <unistd.h>
-#include <string.h>
-#include <sys/ipc.h>
-#include <sys/shm.h>
-
-
 #include "include/ec_main.h"
-#include "include/ec_error.h"
-
-#ifdef DEBUG
-   #include "include/ec_debug.h"
-#endif
-
 
 #define MAX_BUFFERS 10
 #define HEAD_LEN 8
@@ -40,8 +28,7 @@
 typedef struct {
     char *data;
     int len;
-    int key;
-}Buffer_Data;
+} Buffer_Data;
 
 // global data
 
@@ -63,30 +50,20 @@ void Buffer_Flush(int ID);
 
 int Buffer_Create(int len)
 {
-    int i;
+   int i;
 
-#ifdef DEBUG
-   Debug_msg("Buffer_Create -- %d", len);
-#endif
-
-   for (i=0; i<MAX_BUFFERS; i++)
-   {
-      if (!Buffer_List[i].data)
-      {
-         Buffer_List[i].key = shmget(0,len+HEAD_LEN,IPC_CREAT | 0600);
-         if (Buffer_List[i].key == -1)
-            Error_msg("Can't attach %d bytes of shared memory",len);
-
-         Buffer_List[i].data = (char *)shmat(Buffer_List[i].key,0,0);
+   for (i=0; i<MAX_BUFFERS; i++) {
+      if (!Buffer_List[i].data) {
+         Buffer_List[i].data = (char *) calloc(len+HEAD_LEN, sizeof(char));
          Buffer_List[i].len = len;
-         shmctl(Buffer_List[i].key, IPC_RMID, NULL);
-
-         memset(Buffer_List[i].data,0,HEAD_LEN);
          break;
       }
    }
 
    if (i == MAX_BUFFERS) i=-1;
+
+   DEBUG_MSG("Buffer_Create -- [%d] len %d", i, len);
+
    return i;
 }
 
@@ -94,21 +71,22 @@ int Buffer_Create(int len)
 
 void Buffer_Flush(int ID)
 {
-    if (!Buffer_List[ID].data) return;
-    // Reset offsets
-    memset(Buffer_List[ID].data, 0, HEAD_LEN);
+   if (!Buffer_List[ID].data)
+      return;
+   // Reset offsets
+   memset(Buffer_List[ID].data, 0, HEAD_LEN);
 }
 
 
 
 int Buffer_Get(int ID, void *to_read, int size)
 {
-
    int reprise;
    unsigned long *R_Offset, W_Offset;
    char *data;
 
-   if (!Buffer_List[ID].data) return -1;
+   if (!Buffer_List[ID].data)
+      return -1;
 
    R_Offset = (unsigned long *)Buffer_List[ID].data;
    W_Offset = *(unsigned long *)(Buffer_List[ID].data+4);
@@ -120,16 +98,14 @@ int Buffer_Get(int ID, void *to_read, int size)
 
    reprise =- Buffer_List[ID].len+(*R_Offset)+size;
 
-   if (reprise>(long)W_Offset)
-   {
+   if (reprise>(long)W_Offset) {
        size -= reprise-(W_Offset);
        reprise = W_Offset;
    }
 
    if (reprise<0)
        memcpy((char *)to_read, data+(*R_Offset), size);
-   else
-   {
+   else {
        memcpy((char *)to_read, data+(*R_Offset), size-reprise);
        memcpy(((char *)to_read)+size-reprise, data, reprise);
    }
@@ -137,7 +113,6 @@ int Buffer_Get(int ID, void *to_read, int size)
    *R_Offset=(*R_Offset+size)%Buffer_List[ID].len;
 
    return size;
-
 }
 
 
@@ -145,11 +120,14 @@ int Buffer_Get(int ID, void *to_read, int size)
 
 int Buffer_Put(int ID, void *to_write, int size)
 {
+   int retry = 0;
    int reprise;
    unsigned long *R_Offset, *W_Offset;
    char *data;
 
-   if (!Buffer_List[ID].data) return -1;
+
+   if (!Buffer_List[ID].data)
+      return -1;
 
    R_Offset = (unsigned long *)Buffer_List[ID].data;
    W_Offset = (unsigned long *)(Buffer_List[ID].data+4);
@@ -157,18 +135,29 @@ int Buffer_Put(int ID, void *to_write, int size)
 
    reprise  =- Buffer_List[ID].len + (*W_Offset) + size;
 
-   while((*W_Offset<*R_Offset && *W_Offset+size>=*R_Offset) || reprise>=(long)*R_Offset)
-       usleep(1);
+   while((*W_Offset<*R_Offset && *W_Offset+size>=*R_Offset) || reprise>=(long)*R_Offset) {
+      
+      DEBUG_MSG("Buffer_Put -- %d BUFFER FULL !! buff len [%d] byte lost [%d]", ID, Buffer_List[ID].len, size);
+      
+      if (retry >= 2) {
+         //retry = 0;
+         return 0;
+      }
+      retry++;
+      usleep(500);
+   }
+
+   //retry = 0;
 
    if (reprise<=0)
        memcpy(data+*W_Offset, (char *)to_write, size);
-   else
-   {
+   else {
        memcpy(data+*W_Offset, (char *)to_write, size-reprise);
        memcpy(data, ((char *)to_write)+size-reprise ,reprise);
    }
 
    *W_Offset = (*W_Offset+size)%Buffer_List[ID].len;
+
    return 0;
 
 }
@@ -176,3 +165,6 @@ int Buffer_Put(int ID, void *to_write, int size)
 
 /* EOF */
 
+
+// vim:ts=3:expandtab
+
