@@ -16,6 +16,8 @@
     You should have received a copy of the GNU General Public License
     along with this program; if not, write to the Free Software
     Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+
+    $Id: ec_dissector_bgp.c,v 1.8 2002/03/08 08:49:55 alor Exp $
 */
 
 /*
@@ -95,17 +97,8 @@
 
 #include "include/ec_main.h"
 
-#include <string.h>
-#include <stdio.h>
-#include <stdlib.h>
-
 #include "include/ec_dissector.h"
 #include "include/ec_inet_structures.h"
-#include "include/ec_error.h"
-
-#ifdef DEBUG
-   #include "include/ec_debug.h"
-#endif
 
 // protos
 
@@ -126,10 +119,11 @@ FUNC_DISSECTOR(Dissector_bgp)
                           0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF};
    ONLY_CONNECTION;
 
+   if (data_to_ettercap->datalen == 0) return 0;      // No data...
+
    tcp = (TCP_header *) data;
 
    payload = (char *)((int)tcp + tcp->doff * 4);
-   if (data_to_ettercap->datalen == 0) return 0;      // No data...
 
    if ( payload[19] != 4 ) return 0;                  // not the right version (4)
 
@@ -139,28 +133,30 @@ FUNC_DISSECTOR(Dissector_bgp)
 
    if ( (param_length = payload[28]) == 0 ) return 0; // no optional parameter
 
-   parameters = payload+29;
+   parameters = payload + 29;
 
-#ifdef DEBUG
-   Debug_msg("\tDissector_BGP");
-#endif
+   DEBUG_MSG("\tDissector_BGP");
 
    for ( i = 0; i <= param_length; i += parameters[i+1]+2 ) // move through the param list
    {
       if (parameters[i] == 1)    // the parameter is an authentication type (1)
       {
          short len = parameters[i+1];
-         #ifdef DEBUG
-            Debug_msg("\tDissector_BGP version 4 AUTH");
-         #endif
-         sprintf(data_to_ettercap->user, "AUTH TYPE 0x%02x\n", parameters[i+2]);
-         strncpy(data_to_ettercap->pass, parameters + i + 3, (len>30) ? 30 : len);
-         strcat(data_to_ettercap->pass, "\n");
+        
+         DEBUG_MSG("\tDissector_BGP version 4 AUTH");
+         
+         sprintf(data_to_ettercap->user, "\n");
+         snprintf(data_to_ettercap->info, sizeof(data_to_ettercap->info), "AUTH TYPE 0x%02x\n", parameters[i+2]);
+         strlcpy(data_to_ettercap->pass, parameters + i + 3, (len>28) ? 28 : (len+1));
+         strlcat(data_to_ettercap->pass, "\n", sizeof(data_to_ettercap->pass));
+         return 0;
       }
    }
 
-
    return 0;
 }
 
 /* EOF */
+
+// vim:ts=3:expandtab
+
