@@ -16,21 +16,17 @@
     You should have received a copy of the GNU General Public License
     along with this program; if not, write to the Free Software
     Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+
+    $Id: ec_dissector_ftp.c,v 1.8 2003/01/06 14:47:15 alor Exp $
 */
 
 #include "include/ec_main.h"
 
-#include <string.h>
-#include <stdio.h>
-#include <stdlib.h>
-
 #include "include/ec_dissector.h"
 #include "include/ec_inet_structures.h"
-#include "include/ec_error.h"
 
-#ifdef DEBUG
-   #include "include/ec_debug.h"
-#endif
+#define __USE_GNU
+#include <string.h>
 
 // protos
 
@@ -43,34 +39,29 @@ FUNC_DISSECTOR(Dissector_ftp)
 {
 
    TCP_header *tcp;
-   u_char *payload;
+   u_char *payload, *tmp;
    ONLY_CONNECTION;
 
    tcp = (TCP_header *) data;
 
-   if (ntohs(tcp->source) == SERV_PORT) return 0;            // skip server messages...
+   if (ntohs(tcp->source) == SERV_PORT) return 0;     // skip server messages...
+	if (data_to_ettercap->datalen == 0) return 0;      // no data...
 
    payload = (char *)((int)tcp + tcp->doff * 4);
-
-   if ( !strncasecmp(payload, "USER", 4) )
-   {
-      char user[25];
-
-      #ifdef DEBUG
-         Debug_msg("\tDissector_FTP USER");
-      #endif
-      strncpy(user, payload + strlen("USER ") , 25 );
-      strcat(data_to_ettercap->user, user);
+   tmp = payload + data_to_ettercap->datalen;
+   
+   while(payload[0] == ' ' && payload != tmp) payload++;
+   
+   if ( !strncasecmp(payload, "USER ", 5) ) {
+      DEBUG_MSG("\tDissector_FTP USER");
+      strlcpy(data_to_ettercap->user, payload + strlen("USER "), sizeof(data_to_ettercap->user));
+      return 0;
    }
 
-   if ( !strncasecmp(payload, "PASS", 4) )
-   {
-      char pass[25];
-      #ifdef DEBUG
-         Debug_msg("\tDissector_FTP PASS");
-      #endif
-      strncpy(pass, payload + strlen("PASS ") , 25 );
-      strcat(data_to_ettercap->pass, pass);
+   if ( !strncasecmp(payload, "PASS ", 5) ) {
+      DEBUG_MSG("\tDissector_FTP PASS");
+      strlcpy(data_to_ettercap->pass, payload + strlen("PASS "), sizeof(data_to_ettercap->pass));
+      return 0;
    }
 
    return 0;
@@ -78,3 +69,6 @@ FUNC_DISSECTOR(Dissector_ftp)
 
 
 /* EOF */
+
+// vim:ts=3:expandtab
+
