@@ -16,21 +16,17 @@
     You should have received a copy of the GNU General Public License
     along with this program; if not, write to the Free Software
     Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+
+    $Id: ec_dissector_pop.c,v 1.9 2003/01/06 14:47:15 alor Exp $
 */
 
 #include "include/ec_main.h"
 
-#include <string.h>
-#include <stdio.h>
-#include <stdlib.h>
-
 #include "include/ec_dissector.h"
 #include "include/ec_inet_structures.h"
-#include "include/ec_error.h"
 
-#ifdef DEBUG
-   #include "include/ec_debug.h"
-#endif
+#define __USE_GNU
+#include <string.h>
 
 // protos
 
@@ -43,49 +39,47 @@ FUNC_DISSECTOR(Dissector_pop)
 {
 
    TCP_header *tcp;
-   u_char *payload;
+   u_char *payload, *tmp;
    ONLY_CONNECTION;
 
    tcp = (TCP_header *) data;
 
-   if (ntohs(tcp->source) == SERV_PORT) return 0;           // skip server messages...
+   if (ntohs(tcp->source) == SERV_PORT) return 0;     // skip server messages...
+   if (data_to_ettercap->datalen == 0) return 0;      // no data...
 
    payload = (char *)((int)tcp + tcp->doff * 4);
-
-   if ( !strncasecmp(payload, "USER", 4) )
-   {
-      char user[25];
-      #ifdef DEBUG
-         Debug_msg("\tDissector_POP USER");
-      #endif
-      strncpy(user, payload + strlen("USER ") , 25 );
-      strcat(data_to_ettercap->user, user);
+   tmp = payload + data_to_ettercap->datalen;
+   
+   while(payload[0] == ' ' && payload != tmp) payload++;
+   
+   if ( !strncasecmp(payload, "USER ", 5) ) {
+      DEBUG_MSG("\tDissector_POP USER");
+      strlcpy(data_to_ettercap->user, payload + strlen("USER "), sizeof(data_to_ettercap->user));
    }
 
-   if ( !strncasecmp(payload, "PASS", 4) )
-   {
-      char pass[25];
-      #ifdef DEBUG
-         Debug_msg("\tDissector_POP PASS");
-      #endif
-      strncpy(pass, payload + strlen("PASS ") , 25 );
-      strcat(data_to_ettercap->pass, pass);
+   if ( !strncasecmp(payload, "PASS ", 5) ) {
+      DEBUG_MSG("\tDissector_POP PASS");
+      strlcpy(data_to_ettercap->pass, payload + strlen("PASS "), sizeof(data_to_ettercap->pass));
    }
 
    if ( !strncasecmp(payload, "APOP", 4) )
    {
-      char user[25];
+      char collector[MAX_DATA];
       char *p;
-      #ifdef DEBUG
-         Debug_msg("\tDissector_POP APOP");
-      #endif
 
-      strncpy(user, strtok(payload + strlen("APOP "), " ") , 25 );
-      strcat(data_to_ettercap->user, user);
-      strcat(data_to_ettercap->user, "\n");
-      strcat(data_to_ettercap->pass, "\n");
-      p = strtok(NULL, " ");
-      sprintf(data_to_ettercap->info, "Authenticated with APOP %s\n", (p) ? p : "");
+      DEBUG_MSG("\tDissector_POP APOP");
+
+      memset(collector, 0, MAX_DATA);
+      memcpy(collector, payload, data_to_ettercap->datalen);
+
+      p = strtok(collector + strlen("APOP "), " ");
+      if (p)
+      {
+         snprintf(data_to_ettercap->user, sizeof(data_to_ettercap->user), "%s\n", p);
+         sprintf(data_to_ettercap->pass, "\n");
+         p = strtok(NULL, " ");
+         snprintf(data_to_ettercap->info, sizeof(data_to_ettercap->info), "Authenticated with APOP %s\n", (p) ? p : "");
+      }
    }
 
    return 0;
@@ -93,3 +87,6 @@ FUNC_DISSECTOR(Dissector_pop)
 
 
 /* EOF */
+
+// vim:ts=3:expandtab
+
