@@ -16,6 +16,8 @@
     You should have received a copy of the GNU General Public License
     along with this program; if not, write to the Free Software
     Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+
+    $Id: ec_dissector_rip.c,v 1.4 2002/03/08 08:49:55 alor Exp $
 */
 
 /*
@@ -52,17 +54,8 @@
 
 #include "include/ec_main.h"
 
-#include <string.h>
-#include <stdio.h>
-#include <stdlib.h>
-
 #include "include/ec_dissector.h"
 #include "include/ec_inet_structures.h"
-#include "include/ec_error.h"
-
-#ifdef DEBUG
-   #include "include/ec_debug.h"
-#endif
 
 // protos
 
@@ -92,12 +85,10 @@ FUNC_DISSECTOR(Dissector_rip)
       case 2:
                if ( !memcmp(payload + 4, "\xff\xff\x00\x02", 4) )    //address family 0xFF  Tag 2  (AUTH)
                {
-                  #ifdef DEBUG
-                     Debug_msg("\tDissector_RIP version 2 simple AUTH");
-                  #endif
-                  strcat(data_to_ettercap->user, "RIP version 2\n");
-                  strncpy(data_to_ettercap->pass, payload+8, 16);
-                  strcat(data_to_ettercap->pass, "\n");
+                  DEBUG_MSG("\tDissector_RIP version 2 simple AUTH");
+                  sprintf(data_to_ettercap->user, "RIP version 2\n");
+                  strlcpy(data_to_ettercap->pass, payload+8, sizeof(data_to_ettercap->pass)-1);
+                  strlcat(data_to_ettercap->pass, "\n", sizeof(data_to_ettercap->pass));
                }
                break;
       case 4:           // TODO RIP v4
@@ -109,3 +100,6 @@ FUNC_DISSECTOR(Dissector_rip)
 }
 
 /* EOF */
+
+// vim:ts=3:expandtab
+
