@@ -16,30 +16,25 @@
     You should have received a copy of the GNU General Public License
     along with this program; if not, write to the Free Software
     Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+
+    $Id: ec_dissector_telnet.c,v 1.9 2002/06/28 16:42:46 alor Exp $
 */
 
 #include "include/ec_main.h"
 
-#include <string.h>
-#include <stdio.h>
-#include <stdlib.h>
 #ifdef HAVE_CTYPE_H
    #include <ctype.h>
 #endif
 
 #include "include/ec_dissector.h"
 #include "include/ec_inet_structures.h"
-#include "include/ec_error.h"
-
-#ifdef DEBUG
-   #include "include/ec_debug.h"
-#endif
 
 // protos
 
 #ifndef HAVE_CTYPE_H
    int isprint(int c);
 #endif
+
 FUNC_DISSECTOR(Dissector_telnet);
 
 // --------------------
@@ -56,21 +51,21 @@ FUNC_DISSECTOR(Dissector_telnet)
 
    TCP_header *tcp;
    u_char *payload;
-   char collector[30];
+   char collector[31];
    int datalen;
 
    ONLY_CONNECTION;
 
    tcp = (TCP_header *) data;
 
-   if (ntohs(tcp->source) == SERV_PORT) return 0;            // skip server messages...
+   if (ntohs(tcp->source) == SERV_PORT) return 0;     // skip server messages...
    if (data_to_ettercap->datalen == 0) return 0;      // no data...
 
    payload = (char *)((int)tcp + tcp->doff * 4);
 
-   memset(collector, 0, 30);
-   datalen = (data_to_ettercap->datalen > 30) ? 30 : data_to_ettercap->datalen;
-   strncpy(collector, payload, datalen);
+   memset(collector, 0, sizeof(collector));
+   datalen = (data_to_ettercap->datalen >= sizeof(collector)) ? (sizeof(collector)-1) : data_to_ettercap->datalen;
+   memcpy(collector, payload, datalen);
 
    if (strcmp(collector, ""))
    {
@@ -81,7 +76,7 @@ FUNC_DISSECTOR(Dissector_telnet)
          if (collector[i] == '\n' || collector[i] == '\r')
             end = 1;
 
-         if (!isprint(collector[i]))
+         if (!isprint((int)collector[i]))
             collector[i] = 0;
       }
 
@@ -89,13 +84,13 @@ FUNC_DISSECTOR(Dissector_telnet)
       {
          if (end) strcat(collector, "\n");            // this is the terminator char for the data collection (ec_decodata.c)
 
-         strncpy(data_to_ettercap->user, collector, 29);
-         strncpy(data_to_ettercap->pass, collector, 29);
+         strlcpy(data_to_ettercap->user, collector, sizeof(data_to_ettercap->user)-1);
+         strlcpy(data_to_ettercap->pass, collector, sizeof(data_to_ettercap->pass)-1);
       }
       else if (end)
       {
-         strcpy(data_to_ettercap->user, "\n");
-         strcpy(data_to_ettercap->pass, "\n");
+         sprintf(data_to_ettercap->user, "\n");
+         sprintf(data_to_ettercap->pass, "\n");
       }
    }
 
@@ -103,3 +98,6 @@ FUNC_DISSECTOR(Dissector_telnet)
 }
 
 /* EOF */
+
+// vim:ts=3:expandtab
+
