@@ -17,15 +17,11 @@
     along with this program; if not, write to the Free Software
     Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 
+    $Id: ec_inet_forge.c,v 1.7 2002/05/10 20:09:15 alor Exp $
 */
 
 #include "include/ec_main.h"
 #include "include/ec_inet_structures.h"
-#include "include/ec_error.h"
-
-#include <stdio.h>
-#include <stdlib.h>
-#include <string.h>
 
 // protos....
 
@@ -40,7 +36,6 @@ int Inet_Forge_udp( u_char *buf, u_short sp, u_short dp, u_char *data, int data_
 int Inet_Forge_Insert_TCPOpt( u_char *buf, u_char *options, int optlen );
 u_short Inet_Forge_ChecksumIP(u_short *buffer, int size);
 u_short Inet_Forge_Checksum(u_short *buffer, u_short proto, u_short size, u_long IPsource, u_long IPdest);
-void Inet_Forge_RemoveTimeStamp(char *opt, int len);
 u_long Inet_Forge_CRC(u_char *buffer, int len);
 
 //---------------------------------------
@@ -55,7 +50,7 @@ u_char * Inet_Forge_packet( u_short size )
 
    buf = (u_char *)calloc(size+50, sizeof(u_char));
    if (buf == NULL)
-      Error_msg("ec_Inet_Forge:%d calloc() | ERRNO : %d | %s", __LINE__, errno, strerror(errno));
+      ERROR_MSG("calloc()");
 
    return (buf);
 }
@@ -64,7 +59,11 @@ u_char * Inet_Forge_packet( u_short size )
 int Inet_Forge_packet_destroy( u_char *buf )
 {
 
-   if (buf != NULL) free(buf);
+   if (buf != NULL)
+   {
+      free(buf);
+      buf = NULL;
+   }
 
    return 0;
 
@@ -95,9 +94,9 @@ int Inet_Forge_arp( u_char *buf, u_short op, u_char *sa, u_long sip, u_char *da,
    arp.pa_len        = 4;                      // protocol address length
    arp.opcode        = htons(op);              // opcode
    memcpy(arp.source_add, sa, 6);              // source mac
-   *(u_long *)arp.source_ip = sip;             // source ip
+   memcpy(&arp.source_ip, &sip, 4);             // source ip
    memcpy(arp.dest_add, da, 6);                // dest mac
-   *(u_long *)arp.dest_ip = dip;               // dest ip
+   memcpy(&arp.dest_ip, &dip, 4);               // dest ip
 
    memcpy(buf, &arp, sizeof(arp));
 
@@ -112,7 +111,7 @@ int Inet_Forge_ip( u_char *buf, u_long src, u_long dst, u_short len, u_short ide
 
    ip.h_len           = 5;                             // 20 byte header
    ip.version         = 4;                             // version 4
-   ip.tos             = 0;                             // IP tos
+   ip.tos             = 7;                             // IP tos     // we need 7 to enable GRE sniffing/injecting
    ip.t_len           = htons(IP_HEADER + len);        // total length
    ip.ident           = htons(ident);                  // IP ident
    ip.frag_and_flags  = htons(f_f);                    // fragmentation flags
@@ -269,7 +268,7 @@ u_short Inet_Forge_Checksum(u_short *buffer, u_short proto, u_short size, u_long
 
    len = size;
 
-   while(size >1)
+   while( size > 1 )
    {
       cksum += *buffer++;
       size -= sizeof(u_short);
@@ -282,7 +281,7 @@ u_short Inet_Forge_Checksum(u_short *buffer, u_short proto, u_short size, u_long
    cksum += htons(proto);
    cksum += htons(len);
 
-   if(size ) cksum += *(u_char*)buffer;
+   if ( size ) cksum += *(u_char*)buffer;
 
    cksum = (cksum >> 16) + (cksum & 0xffff);
    cksum += (cksum >> 16);
@@ -291,25 +290,6 @@ u_short Inet_Forge_Checksum(u_short *buffer, u_short proto, u_short size, u_long
 }
 
 
-void Inet_Forge_RemoveTimeStamp(char *opt, int len)
-{
-    char *arrive;
-    arrive = opt + len;
-    while(opt < arrive)
-    {
-         if (*opt==8)
-         {
-            memset(opt,1,10);
-            opt += 10;
-         }
-         else
-            if (*opt == 1) opt++;
-            else
-               if (*opt > 1) opt += opt[1];
-    }
-}
-
-
 u_long Inet_Forge_CRC(u_char *buffer, int len)
 {
     unsigned long crc_32_tab[] = {
@@ -358,3 +338,6 @@ u_long Inet_Forge_CRC(u_char *buffer, int len)
 
 
 /* EOF */
+
+// vim:ts=3:expandtab
+
